#include "format.h"
#include <stdio.h>
#include <math.h>

#define TEST( a, b ) \
 aa = a; b; \
 bb = buffer; \
 cout << aa << "\t==\t" << bb << "\t: "; \
 if( aa == bb ) cout << "true"; else cout << "false"; \
 cout << endl;

using namespace std;

int main()
{
  char buffer[250];
  std::string aa, bb;

#ifndef NFORMAT
  try {
#endif

    TEST( format ("%s", "String" ),                 sprintf( buffer, "%s", "String" ));
    TEST( format ("%d", 155 ),                      sprintf( buffer, "%d", 155 ));
    TEST( format ("%f", 155.1 ),                    sprintf( buffer, "%f", 155.1 ) );
    TEST( format ("%f", (double) 155.1 ) ,          sprintf( buffer, "%f", (double) 155.1 ) );
    TEST( format ("%x", 1515 ),                     sprintf( buffer, "%x", 1515 ) );
    TEST( format ("%X", 1515 ),                     sprintf( buffer, "%X", 1515 ) );
    TEST( format ("%05d", 3 ),                      sprintf( buffer, "%05d", 3 ) ); 
    TEST( format ("%05.f", 3.32 ),                  sprintf( buffer, "%05.f", 3.32 ) );
    TEST( format ("%#5x", 1515 ),                   sprintf( buffer, "%#5x", 1515 ) ); 
    TEST( format ("%#5X", 1515 ),                   sprintf( buffer, "%#5X", 1515 ) ); 
    TEST( format ("%0*2$d", 1515, 6 ),              sprintf( buffer, "%0*2$d", 1515, 6 ) ); 
    TEST( format ("%0*d", 6, 1515 ),                sprintf( buffer, "%0*d", 6, 1515 ) ); 
    TEST( format ("%*d", 6, 1515 ),                 sprintf( buffer, "%*d", 6, 1515 ) ); 
    TEST( format ("%2$*1$d", 6, 1515 ),             sprintf( buffer, "%2$*1$d", 6, 1515 ) ); 
    TEST( format ("%2$d%1$d", 6, 1515 ),            sprintf( buffer, "%2$d%1$d", 6, 1515 ) ); 
    TEST( format ("%.*f", 6, 32.32 ),               sprintf( buffer, "%.*f", 6, 32.32 ) ); 
    TEST( format ("%.*2$f", 15.15, 5 ),             sprintf( buffer, "%.*2$f", 15.15, 5 ) );  
    TEST( format ("%.*f", 5, 15.15 ),               sprintf( buffer, "%.*f", 5, 15.15 ) ); 
    TEST( format ("%.3s", "HALLO" ),                sprintf( buffer, "%.3s", "HALLO" ) ); 
    TEST( format ("pi = %.5f", 4 * atan(1.0)),      sprintf( buffer, "pi = %.5f", 4 * atan(1.0)) );  
    TEST( format( "%*d-%*d-%*d", 1, 3, 2, 4, 5, 6), sprintf( buffer, "%*d-%*d-%*d", 1, 3, 2, 4, 5, 6) );
    TEST( format( "[%#8.3x]", 0x42 ),               sprintf( buffer, "[%#8.3x]", 0x42 ) );
    TEST( format( "[%#08x]", 0x42 ),                sprintf( buffer, "[%#08x]", 0x42 ) );
    TEST( format( "[%#8.3o]", 0x42 ),               sprintf( buffer, "[%#8.3o]", 0x42 ) );
    TEST( format( "[%#08o]", 0x42 ),                sprintf( buffer, "[%#08o]", 0x42 ) );
    TEST( format( "[%-#08x]", 0x42 ),                sprintf( buffer, "[%-#08x]", 0x42 ) );
    TEST( format( "[%-#08o]", 0x42 ),                sprintf( buffer, "[%-#08o]", 0x42 ) );
    
    TEST( format( "%2$s %1$s", "A", "B" ),          sprintf( buffer, "%2$s %1$s", "A", "B" ) ); 
    TEST( format( "[%]", "A" ),                       sprintf( buffer, "[%]", "A" ) ); 
    TEST( format( "%", "A" ),                       sprintf( buffer, "%", "A" ) ); 

    cout << endl;

#ifndef NFORMAT
    // format only test, don't try this with printf
    // format will crash too, if NFORMAT is defined
    cout << format( "%s", 5 ) << "\t: true" << endl;

  } catch ( Format::Error &err )
    {
      cout << "Error: " << err.err << endl;
    }
#endif


  /** example for using the Format::PrintF class **/

  Format::PrintF<std::ostream> out( std::cout );

  out( "hello\n" );
  out( "hello %s\n", "Mr Spock" );
  
  /** The class is nice for printing out debugging informations.
      You can set specific Modules, and levels
  */
  
  enum Module
      {
	  MODULE_ALL = -1, // -1 is the default value for all modules
	  MODULE_ONE = 1,
	  MODULE_TWO,
	  MODULE_THREE
      };

  out.set_module( MODULE_ALL );
  
  out( MODULE_ONE ) << "message from ONE\n";
  out( MODULE_TWO ) << "message from TWO\n"; 

  out.set_module( MODULE_ONE );

  out( MODULE_ONE ) << "message from ONE\n";
  out( MODULE_TWO ) << "message from TWO\n"; // won't be shown
  
  /** next step,  debug levels **/

  out.set_debug_level( 2 ); // all messages, lower than 3 to will be printed out
  out.set_module( MODULE_ALL );

  out[0] << "level 0\n";
  out[1] << "level 1\n";
  out[2] << "level 2\n";
  out[3] << "level 3\n"; // will be ignored

  /** combination of level and module **/

  out.set_module( MODULE_ONE );

  out(MODULE_ALL)[0]( "ALL level 0\n" );      // ignored only MODULE_ONE will be printed
  out(MODULE_ONE)[1] << "ONE level 1\n";      
  out(MODULE_TWO)[2] << "TWO level 2\n";      // ignored, wrong module
  out(MODULE_THREE)[3] << "THREE level 3\n";  // ignored, wrong module
  out(MODULE_ONE)[3] << "ONE level 4\n";      // ignored, wrong debug level

  return 0;
}
